
package Class::MOP::Method::Meta;

use strict;
use warnings;

use Carp         'confess';
use Scalar::Util 'blessed';

our $VERSION   = '1.08';
$VERSION = eval $VERSION;
our $AUTHORITY = 'cpan:STEVAN';

use constant DEBUG_NO_META => $ENV{DEBUG_NO_META};

use base 'Class::MOP::Method';

sub _generate_meta_method {
    my $method_self = shift;
    my $metaclass   = shift;
    sub {
        # this will be compiled out if the env var wasn't set
        if (DEBUG_NO_META) {
            my ($self) = @_;
            # it's okay if we installed a meta method in a subclass of a class
            # with a legitimate meta method (we'll still die if we install a
            # meta method in a subclass of a class we installed a meta method
            # in, since this function is itself defined in Class::MOP)
            #if (my $meta = try { $self->SUPER::meta }) {
                #return $meta if $meta->isa('Class::MOP::Class');
            #}
            # it's okay if the test itself calls ->meta, we only care about if
            # the mop internals call ->meta
            confess "'meta' method called by MOP internals"
                if caller =~ /^(?:Class::MOP|metaclass)(?:::|$)/;
        }
        # we must re-initialize so that it
        # works as expected in subclasses,
        # since metaclass instances are
        # singletons, this is not really a
        # big deal anyway.
        $metaclass->initialize(blessed($_[0]) || $_[0])
    };
}

sub wrap {
    my ($class, @args) = @_;

    unshift @args, 'body' if @args % 2 == 1;
    my %params = @args;
    confess "Overriding the body of meta methods is not allowed"
        if $params{body};

    my $metaclass_class = $params{associated_metaclass}->meta;
    $params{body} = $class->_generate_meta_method($metaclass_class);
    return $class->SUPER::wrap(%params);
}

1;

__END__

=pod

=head1 NAME

Class::MOP::Method::Meta - Method Meta Object for C<meta> methods

=head1 DESCRIPTION

This is a L<Class::MOP::Method> subclass which represents C<meta>
methods installed into classes by Class::MOP.

=head1 METHODS

=over 4

=item B<< Class::MOP::Method::Wrapped->wrap($metamethod, %options) >>

This is the constructor. It accepts a L<Class::MOP::Method> object and
a hash of options. The options accepted are identical to the ones
accepted by L<Class::MOP::Method>, except that C<body> cannot be passed
(it will be generated automatically).

=back

=head1 AUTHORS

Jesse Luehrs E<lt>doy at tozt dot netE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2006-2010 by Infinity Interactive, Inc.

L<http://www.iinteractive.com>

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

